# All or portions of this script subject to the following copyrights:
# Copyright (c) 2002       Riverbank Computing Limited <info@riverbankcomputing.co.uk>
# Copyright (c) 2002-2003  Jim Bublitz <jbublitz@nwinternet.com>
# Copyright (c) 2003       Hans-Peter Jansen <hpj@urpla.net>

# This software is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this library; see the file COPYING.
# If not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


# This is the build script for PyKDE.  It should be run in the top level
# directory of the source distribution and by the Python interpreter for which
# it is being built.  It uses qmake to do the hard work of generating the
# platform specific Makefiles.

import sys
import os
import os.path
import string
import re
import getopt
import tempfile
import time
import shutil
import py_compile

sys.path.append ("build")

from discover import *
from config import *

False = 0
True  = 1

# globals

dis             = None    # discovery object

nrThreads       = 0       # number of threads
catSplit        = 1
progName        = ""      # program name
opts            = {}      # dictionary of option:arg from command line

proPatches      = {}
makefilePatches = {}


internalError = \
"""

An internal error occured.  Review the installation documentation or
report all the output from the program, including the traceback,
to the PyKDE mailing list: <pykde@mats.imk.fraunhofer.de>. Thanks.

"""

# utilities ----------------------------------------------------------

def format (mess, err = False):
    """
    Format a message by inserting line breaks at appropriate places.

    mess is the text of the message.

    Return the formatted message.
    """
    msg    = ''
    margin = 78
    curs   = 0
    if err:
        msg = "*" * margin + "\n"

    for w in string.split (mess):
        l = len(w)
        if curs != 0 and curs + l > margin:
            msg = msg + "\n"
            curs = 0

        if curs > 0:
            msg = msg + " "
            curs = curs + 1

        msg = msg + w
        curs = curs + l

    if err:
        msg = msg + "\n" + ("*" * margin) + "\n"
    else:
        msg = msg + '\n'

    return msg

def inform (mess):
    """
    Display an information message.

    mess is the text of the message.
    """
    sys.stdout.write (format (mess))
    sys.stdout.flush ()

def runProgram (prog,argv = [], fatal = 1, wd = None):
    """
    Execute a program.

    prog is the name of the program.
    argv is the optional list of program arguments.
    fatal is non-zero if an error is considered fatal.
    wd is an optional build directory.

    Returns the error code.
    """
    run = prog + ' ' + string.join(argv)
    if wd:
        # FIXME: reason, why theading will not work on win32 yet <hpj@urpla.net>
        run = 'cd ' + wd + ' >/dev/null && ' + run
    #inform("run: <%s>" % run)
    rc = os.system (run)

    if rc != 0 and fatal:
        inform("%s failed with an exit code of %d." % (prog,rc))
        raise   # force traceback and exit

    return rc


def runMake (target = None, fatal = 1):
    """
    Run make.

    target is the optional make target.
    fatal is non-zero if an error is considered fatal.

    Returns the error code.
    """
    if target is None:
        argv = []
    else:
        argv = [target]

    return runProgram (dis.make, argv, fatal)


def copyToFile (name, text):
    """
    Copy a string to a file.

    name is the name of the file.
    text is the contents to copy to the file.
    """
    f = open (name, "w")
    f.write (text)
    f.close ()


def patchFile (name, pdict, bakfile = None):
    """
    Apply the patches to a file.

    name is the name of the file.
    pdict is the patch dictionary.
    bakfile is the name of the optional backup file for the unpatched version.
    """
    newname = name + ".new"

    fi = open (name, "r")
    fo = open (newname, "w")

    for line in fi.readlines ():
        for (pattern, repl) in pdict.values ():
            line = re.sub (pattern, repl, line)

        fo.write (line)

    fi.close ()
    fo.close ()

    if bakfile is None:
        os.remove (name)
    else:
        os.rename (name,bakfile)

    os.rename (newname,name)

def pushDir (newdir):
    """
    Change to a new directory and return the old one.

    Note: using this function is not thread safe!

    newdir is the new directory.
    """
    olddir = os.getcwd ()
    os.chdir (newdir)

    return olddir


def popDir (olddir):
    """
    Change back to a previous directory.

    Note: using this function is not thread safe!

    olddir is the old directory.
    """
    os.chdir (olddir)


def buildMakefile (pro, _propat = None, _makepat = None, wd = None):
    """Run qmake or tmake to create a Makefile from a project file.

    pro is the name of the project file.
    _propat is an optional project patch dictionary.
    _makepat is an optional project patch dictionary.
    wd is an optional build directory.
    """


    if not _propat:
        _propat = proPatches
    if not _makepat:
        _makepat = makefilePatches

    if pro == "pythonize.pro":
        _propat ["LIBPYTHON"] = [re.compile("@BL_LIBPYTHON@",re.M), dis.pyLib.path + " -lpython%s" % (dis.py.vers)]
        _propat ["INCLUDEPATH"] = [re.compile ("@BL_INCLUDEPATH@", re.M), dis.pyIncl.path]

    # Preserve the original project file.
    pf = addBuildDir(pro,wd)
    probak = pf + ".bak"
    patchFile (pf, _propat, probak)

    # Make sure that the Makefile has a later timestamp than the project file.
    time.sleep (1)

    runProgram (dis.qmake,['-o', 'Makefile', pro],1,wd)
    os.remove (pf)
    os.rename (probak,pf)

    patchFile (addBuildDir ("Makefile",wd), _makepat)


def addBuildDir (file, wd):
    """Add an optional build path to a given file.

    file is the file name.
    wd is an optional build directory.

    returns the file name.
    """
    if wd:
        file = os.path.join (wd, file)
    return file

#------------------------------------------------------------------

def initGlobals ():
    global platCopy, platMove, platRemove, platMkdir, platRmdir, catCppFiles

    if sys.platform == "win32":
        error ("Windows not currently supported")

    platCopy   = "cp"
    platMove   = "mv"
    platRemove = "rm -f"
    platMkdir  = "mkdir -p"
    platRmdir  = "rm -rf"

    # concatenate files for compiling (sip only)
    if opts.has_key ("-c"):
        if opts ["-c"] == "+":
            catCppFiles = True
        elif opts ["-c"] == "-":
            catCppFiles = True
        else:
            error ("Invalid argument for -c switch ('+' or '-')")

    if opts.has_key ("-j"):
        try:
            catSplit = int (opts ["-j"])
        except:
            usage()

def installChecks ():
    """
    Carry out basic checks about the installation.
    """
    inform ("Building PyKDE %s on %s." % (Release, sys.platform))
    print

    # Check we are in the top level directory.
    if not os.access (progName,os.F_OK):
        error ("This program must be run from the top level directory of the package, ie. the directory containing this program")

    # create ./libs if it doesn't exist
    if not os.access (libDir, os.F_OK) \
        and runProgram (platMkdir, [libDir]) \
        and not os.access(libDir,os.F_OK):
        error ("The %s temporary library destination directory cannot be created." % (libDir))

    # This is really the temporary destination (libDir or ../libs)
    proPatches ["DESTDIR"] = [re.compile ("@BL_DESTDIR@",re.M), os.path.join ("..", libDir)]

    defines = ""
    proPatches ["DEFINES"] = [re.compile ("@BL_DEFINES@",re.M), defines]


def mkTempBuildDir (olddir = None):
    """Create a temporary build directory for a console application called
    qttest, complete with patched Makefile.  The global tempBuildDir is set to
    the name of the directory.  The temporary directory becomes the current
    directory.

    olddir is None if the directory should be created, otherwise it is deleted.

    Returns the name of the previous current directory.
    """
    global tempBuildDir

    if olddir is None:
        tempBuildDir = tempfile.mktemp ()

        try:
            os.mkdir (tempBuildDir)
        except:
            error ("Unable to create temporary directory")

        newdir = pushDir (tempBuildDir)

        copyToFile ("qttest.pro",
"""TEMPLATE = app
TARGET = qttest
CONFIG = qt console warn_off release @BL_THREAD@
INCLUDEPATH = @BL_INCLUDEPATH@
DEFINES = @BL_DEFINES@
SOURCES = qttest.cpp

""")

        # Create a dummy source file to suppress a qmake warning.
        copyToFile ("qttest.cpp","")
        buildMakefile ("qttest.pro")

    else:
        popDir (olddir)
        newdir = None

        shutil.rmtree (tempBuildDir, 1)

    return newdir

def compileChecks ():
    """
    Tune the compiler flags, and see which PyQt modules to build.
    """
    # See if we can re-run make with some preferred options.  Don't do it for
    # Windows because we know the options aren't supported - and MSVC only
    # issues a warning so we can't detect the result properly anyway.
    copyToFile ("qttest.cpp",
"""int main(int argc,char **argv)
{
    return 0;
}
""")

    inform ("Checking to see if the C++ compiler supports -fno-exceptions.")

    tmpPatches = {"CXXFLAGS": [re.compile("(^CXXFLAGS.*)$",re.M), "\\1 -fno-exceptions\n"]}
    patchFile ("Makefile",tmpPatches,"Makefile.bak")

    if runMake (None, 0):
        # Restore the old working Makefile.
        os.remove ("Makefile")
        os.rename ("Makefile.bak","Makefile")

        inform ("The C++ compiler doesn't support -fno-exceptions.")
    else:
        os.remove ("Makefile.bak")

        makefilePatches ["CXXFLAGS"] = [re.compile ("(^CXXFLAGS.*)$", re.M), "\\1 -fno-exceptions\n"]

        inform ("The C++ compiler supports -fno-exceptions.")
        print


def generateFeatures(featfile):
    """
    Generate the header file describing the Qt features that are enabled if
    it doesn't already exist.  (If it already exists then we are probably cross
    compiling and generated the file through other means.)

    featfile is the name of the features file.
    """
    if os.access (featfile,os.F_OK):
        inform ("Using existing features file.")
        print
        return

    runMake ("clean")

    inform ("Generating the features file.")

    # The features that a given Qt configuration may or may not support.
    flist = ["ACTION", "CLIPBOARD", "CODECS", "COLORDIALOG", "DIAL", "DNS",
             "DOM", "DRAGANDDROP", "ICONVIEW", "IMAGE_TEXT", "INPUTDIALOG",
             "FILEDIALOG", "FONTDATABASE", "FONTDIALOG", "MESSAGEBOX",
             "MIMECLIPBOARD", "NETWORKPROTOCOL", "PICTURE", "PRINTDIALOG",
             "PRINTER", "PROGRESSDIALOG", "PROPERTIES", "SEMIMODAL",
             "SIZEGRIP", "SOUND", "SPLITTER", "STYLE_CDE", "STYLE_INTERLACE",
             "STYLE_MOTIF", "STYLE_MOTIFPLUS", "STYLE_PLATINUM", "STYLE_SGI",
             "STYLE_WINDOWS", "TABDIALOG", "TABLE", "TABLEVIEW",
             "TRANSFORMATIONS", "TRANSLATION", "WIZARD", "WORKSPACE"]

    # Generate the program which will generate the features file.
    f = open ("qttest.cpp","w")

    f.write (
"""#include <stdio.h>
#include <Python.h>
#include <qglobal.h>
#include <qapplication.h>

int main(int argc,char **argv)
{
    FILE *fp;
    QApplication app(argc,argv,0);

    if ((fp = fopen(\"%s\",\"w\")) == NULL)
        return 1;

#if !defined(QT_THREAD_SUPPORT) || !defined(WITH_THREAD)
    fprintf(fp,"-x Qt_THREAD_SUPPORT\\n");
#endif
""" % (featfile))

    for feat in flist:
        f.write (
"""
#if defined(QT_NO_%s)
    fprintf(fp,"-x Qt_%s\\n");
#endif
""" % (feat, feat))

    f.write(
"""
    fclose(fp);

    return 0;
}
""")

    f.close ()

    runMake ()
    os.system ("pwd")
    runProgram ("./qttest")

    inform ("Generated the features file.")
    print

def generateCLibs (mname):
    # Generate the Makefile.
    inform("Generating the Makefile for the %s module." % (mname))

    if string.find (mname,"test") >= 0:
        target = "testPythonize"
    else:
        target = mname + "cmodule"

    wd = mname

    runProgram ("rm", ["-f", os.path.join (wd, "*.o")])

    if mname == "pykpanelapplet":
        runProgram ("rm", ["-f", os.path.join (wd, "moc*")])

    elif mname == "pythonize/testPythonize":
        runProgram ("rm", ["-f", os.path.join (wd, "testPythonize")])

    # customized in config.py
    global proPatches, makefilePatches
    addCLibProPatches (dis, proPatches)
    addCLibMakefilePatches (dis, makefilePatches)

    # deep copy the patch dicts for threading safety
    _makepat = makefilePatches.copy()
    _propat  = proPatches.copy()

    _propat ["TARGET"] = [re.compile ("@BL_TARGET@",re.M), target]

    # creates the make files
    if target == "testPythonize":
        buildMakefile (target + ".pro", _propat, _makepat, wd)
    else:
        buildMakefile (mname + ".pro", _propat, _makepat, wd)

    # edits the make files
    fixCLibInstallTarget (mname)

def fixCLibInstallTarget (mname = None):
    """
    Fix the install target for a Makefile.

    mname is the name of the module
    """

    if not mname or string.find (mname, "test") >=0:
        return

    # The correct path for use in the install commands
    # is <module name>/Makefile - the previous version
    # was omitting the module name, so all mods ended
    # up being done to the top level Makefile
    fn      = os.path.join (mname, "Makefile")
    newFn   = os.path.join (mname, "Makefile.new")
    libWC   = "lib%s*" % (mname)
    libName = "lib%s" % (mname)

    fi     = open(fn,"r")
    fo     = open(newFn,"w")
    buff  = fi.read ()

    installTxt = ("\ninstall:\n")

    # <module name>py and .pyc are created in the <module name> dir
    # (eg dcop.py is created in ../dcop)
    # All libs are written to ,,/libs - this code just causes
    # 'make install' to copy all of the relevant files to the
    # correct end destination (normally python/site-packages)
    # This allows 'make' to be run as a user but 'make install'
    # still requires root accesss, at least for site-packages
    # (as it should)
    if mname == "pykpanelapplet":
        lnDir      = dis.kdeLib.path
        installTxt = installTxt\
                     +"\t-%s %s %s\n" % (platMove, os.path.join ("..", libDir, libWC),\
                       os.path.join (dis.kdeLib.path, "kde3"))\
                     + "\t-chown 0:0 %s\n" % (os.path.join (dis.kdeLib.path, "kde3", libWC))

        lnSrc = os.path.join (dis.kdeLib.path, "kde3", libName + ".so.1.0.0")
        lnDst = os.path.join (lnDir, libName)

        installTxt = installTxt + generateLinks (lnSrc, lnDst)

        linkUp = ["qt", "qtxml", "dcop", "kdecore", "kdefx", "kdeui", "sip"]
        for d in linkUp:
            isLibSip = d == "sip"
            if not isLibSip:
                lnSrc = os.path.join (dis.installDir.path, "lib%scmodule.so.1.0.0" % d)
                lnDst = os.path.join (lnDir, "lib%scmodule" % d)
            else:
                lnSrc = dis.installDir.path
                lnDst = lnDir


            installTxt = installTxt + generateLinks (lnSrc, lnDst, isLibSip)

        cleanTxt    = "clean:\n\t-%s %s\n\t-" % (platRemove, os.path.join ("..", libDir, libWC))

        installTxt, ignore, lnSrc, lnDst = customCLibInstall (dis, "wabbit", installTxt)

    elif mname == "pythonize":
        installTxt = installTxt\
                    + "\t-%s %s %s\n" % (platMove, os.path.join ("..", libDir, libWC), dis.installDir.path)\
                    + "\t-chown 0:0 %s\n" % (os.path.join (dis.installDir.path, libWC))

        cleanTxt    = "clean:\n\t-%s %s\n\t-" % (platRemove, os.path.join ("..", libDir, libWC))

        lnSrc = os.path.join (dis.installDir.path, libName + ".so.1.0.0")
        lnDst = os.path.join (sys.prefix, "lib", libName)

        installTxt = installTxt + generateLinks (lnSrc, lnDst)

    else:
        installTxt, cleanTxt, lnSrc, lnDst = customCLibInstall (dis, mname, installTxt)
        if lnSrc and lnDst:
            installTxt = installTxt + generateLinks (lnSrc, lnDst)


    # force a ldconfig so linked modules can be found
    # some distributions don't put /sbin in $PATH
    if mname == manifest [-1]:
        installTxt = installTxt + '\t-echo "Please wait ..."\n\t-/sbin/ldconfig\n'

    buff = re.compile ("^clean: ", re.M).sub (cleanTxt, buff)
    buff = re.compile ("install: all").sub (installTxt, buff)

    fo.write (buff)
    fi.close()
    fo.close()
    os.unlink (fn)
    os.rename (newFn, fn)

def generateLinks (lnSrc, lnDst, isLibSip = False):
    links = ""

    if isLibSip:
        libvlist = glob.glob (os.path.join (lnSrc, "libsip*"))
        links = links + "\t-rm -f %s\n" %(os.path.join (lnDst,"libsip.*"))
        for lib in libvlist:
            target = os.path.join (lnDst, os.path.basename (lib))
            links  = links + "\t-ln -s %s %s\n" % (lib, target)

    else:
        libvlist = [".so", ".so.1", ".so.1.0", ".so.1.0.0"]
        links    = links + "\t-rm -f %s\n" % (lnDst + "*")
        for suffix in libvlist:
            links = links + "\t-ln -s %s %s\n" % (lnSrc, lnDst + suffix)

    return links

def generateSipLib (mname, platTag, qtTag, xtrTag = None, cpu_pool = None):
    """Generate the C++ source code for a particular PyQt module.

    mname    is the name of the module.
    plattag  is the SIP tag for the platform.
    qttag    is the SIP tag for the Qt version.
    xtrtag   is an optional extra SIP tag.
    cpu_pool is an optional bounded semaphore to control threading.
    """
    if cpu_pool:
        cpu_pool.acquire ()

    inform ("Generating the C++ source for the %s module." % (mname))

    libWC = "lib%s*" % (mname)

    try:
        shutil.rmtree (mname)
        if runProgram (platRmdir, [os.path.join ("..", libDir, libWC)]) != 0:
            error ("Unable to delete old library files in %s" % (libDir))
    except:
        pass

    try:
        os.mkdir (mname)
    except:
        error ("Unable to create the %s directory" % (mname))

    kv = "%s%s%s" % (dis.kde.major, dis.kde.minor, dis.kde.micro)
    if not kv in KDEVers:
        for i in range (len (KDEVers) - 1, -1, -1):
            if KDEVers [i] < kv:
                kv = KDEVers [i]
                break

    kdeVersion = "-kde%s" % kv
    kdeTag     = "KDE_%s_%s_%s" % (kv [0], kv [1], kv [2])
    pro = mname + ".pro"
    fullmname = mname +  kdeVersion + ".sip"


    # platTag          is Windows, Linux (X11), MacOS (>)
    # qtTag and kdeTag are Qt and KDE versions
    # features         is the features file
    # sip              is the source (sip) file subdirectory
    # pyQtSipDir       is where the PyQt sip files are
    # mname            is the module name (-m for generating
    #                  the make files, -c for the actual module name)
    # fullname         is the name of the top level sip file
    #                  that controls sip file inclusion, imports, etc.
    argv = ["-t", platTag,
            "-t", qtTag,
            "-t", kdeTag,
            "-z", "features",
            "-I", "sip",
            "-I", dis.pyQtSip.path,
            "-m", mname + "/" + pro,
            "-c", mname,
            fullmname]

    if xtrTag:
        argv.insert(0,xtrtag)
        argv.insert(0,"-t")

#    print qttag, kdetag

    runProgram (dis.sipBin.bin, argv)
    runProgram (sys.executable + " postproc", ['-p', mname, "-o", "tr", "*.cpp"])

    if modInfo [mname][1]:
        runProgram (sys.executable + " postproc", modInfo [mname][1])

    # Python names binary modules differently on different platforms.
    target = mname + "cmodule"

    # Generate the Makefile.
    inform ("Generating the Makefile for the %s module." % (mname))

    # customized in config.py
    global proPatches, makefilePatches
    addSipLibProPatches (dis, proPatches)
    addSipLibMakefilePatches (dis, makefilePatches)

    # deep copy the patch dicts for threading safety
    _makepat = makefilePatches.copy()
    _propat = proPatches.copy()

    _propat ["TARGET"] = [re.compile ("@BL_TARGET@",re.M), target]

    if catCppFiles:
        catFiles (mname, mname) # module, wd

    # creates the make files
    buildMakefile (pro, _propat, _makepat, mname)

    # edits the make files
    fixSipLibInstallTarget (mname)

    # Compile the Python part of the module.
    pyname = mname + ".py"

    inform ("Compiling %s." % (pyname))
    py_compile.compile (addBuildDir (pyname, mname))
    print

    if cpu_pool:
        cpu_pool.release ()

def catFiles (mname, wd = None):
    """
    Concatenate a modules C++ source files.

    pro is the name of the project file.
    wd is an optional build directory.
    """
    am = ""
    if catSplit > 1:
        am = " (into %d pieces)" % catSplit
    inform ("Concatenating the C++ files for the %s module%s." % (mname, am))

    pro = addBuildDir (mname + ".pro", wd)

    f = open (pro,"r")
    buf = f.read ()
    f.close ()

    pat = re.compile ("^SOURCES =.*.cpp\n", re.M | re.S)
    match = re.search (pat, buf)
    srclist = buf [match.start (0):match.end (0)]
    srclist = srclist [13:-1]
    srclist = string.replace (srclist, "\\\n\t","")
    srclist = string.split (srclist, " ")

    # Concatenate the files.
    p = 0
    plist = []
    nrinpiece = (len (srclist) + catSplit - 1) / catSplit

    #inform("Combining %d modules into %d pieces (%d)" % (len(srclist), catSplit, nrinpiece))

    # Do each piece.
    while srclist:
        pname = "%shuge%d.cpp" % (mname, p)
        plist.append (pname)

        d = open (addBuildDir (pname,wd), "w")

        for cppfile in srclist [:nrinpiece]:
            f = open (addBuildDir (cppfile,wd), "r")
            d.write (f.read ())
            f.close ()

        d.close ()

        srclist = srclist [nrinpiece:]
        p = p + 1

    # Replace the C++ file names in the project file.
    buf = re.sub (pat,"SOURCES = " + string.join (plist," ") + "\n", buf)

    f = open (pro + ".new","w")
    f.write (buf)
    f.close ()

    os.remove (pro)
    os.rename (pro + ".new", pro)

def fixSipLibInstallTarget (mname = None):
    """
    Fix the install target for a Makefile.

    mname is the name of the  module
    """

    if not mname:
        return

    # The correct path for use in the install commands
    # is <module name>/Makefile - the previous version
    # was omitting the module name, so all mods ended
    # up being done to the top level Makefile
    fn      = os.path.join (mname, "Makefile")
    newFn   = os.path.join (mname, "Makefile.new")
    libWC   = "lib%s*" % (mname)
    libName = "lib%s" % (mname)

    fi      = open (fn, "r")
    fo      = open (newFn, "w")
    buff    = fi.read ()

    installTxt = ("\ninstall:\n")
    lnDir      = dis.kdeLib.path

    # <module name>py and .pyc are created in the <module name> dir
    # (eg dcop.py is created in ../dcop)
    # All libs are written to ,,/libs - this code just causes
    # 'make install' to copy all of the relevant files to the
    # correct end destination (normally python/site-packages)
    # This allows 'make' to be run as a user but 'make install'
    # still requires root accesss, at least for site-packages
    # (as it should)
    installTxt = installTxt\
                + "\t-%s %s.py %s\n" % (platCopy,mname,dis.installDir.path)\
                + "\t-%s %s.pyc %s\n" % (platCopy,mname,dis.installDir.path)\
                + "\t-%s %s %s\n" % (platMove, os.path.join ("..", libDir, libWC), dis.installDir.path)\
                + "\t-chown 0:0 %s\n" % (os.path.join (dis.installDir.path, libWC))

    cleanTxt    = "clean:\n\t-%s %s\n\t-" % (platRemove, os.path.join ("..", libDir, libWC))

    installTxt, cleanTxt = customSipLibInstall (dis, mname, installTxt,cleanTxt)

    buff = re.compile ("^clean: ", re.M).sub (cleanTxt, buff)
    buff = re.compile ("install: all").sub (installTxt, buff)

    fo.write (buff)
    fi.close ()
    fo.close ()
    os.unlink (fn)
    os.rename (newFn, fn)


def fileOpts (fn):
    try:
        optfile = open (fn, 'r')
    except:
        error ("Could not open option file %s" % (fn))

    opts = []

    for line in optfile.readlines ():
        if (line [0] == '#') or (line == '\n'):
           continue
        elif line [0] == '-':
            opts.append ((line [0:2], string.strip (line [2:])))
        else:
            opts.append ((line [0:1], string.strip (line [1:])))

    print 'Additional options: ',
    for opt, arg in opts:
        print opt, arg,
    print

    return opts


def getOptions (argv):
    global opts

    try:
        optlist, args = getopt.getopt(argv[1:], shortOptions)
        flag = False
    except getopt.GetoptError:
        usage ()

    # modify the options list via config.py method
    addOptions (optlist, args)

    # Look for '-z' first and process that switch
    # (command line switches override file switches)
    for opt, arg in optlist:
        if opt == "-z":
             optlist = fileOpts (arg) + optlist
             break
        elif opt == "-h":
            usage (0)
    else:
        if args: usage()

    for opt, arg in optlist:
        opts [opt] = arg

def generateSipLibs (siplibs):
    if isPyKDE:
        if nrThreads:
            threads = []
            cpu_pool = BoundedSemaphore (nrThreads)
            inform ("Starting %d builder threads for %s" % \
                    (nrThreads, string.join(siplibs, " ")))

        qtTag = dis.qt.getTag (qttags, "Qt")
        for mod in siplibs:
            if nrThreads:
                t = Thread (target = generateSipLib, name = mod,
                            args = (mod, platformTag,  qtTag, None, cpu_pool))
                threads.append(t)
                t.start()
            else:
                generateSipLib (mod, platformTag, qtTag)

        # wait for threads to finish..
        if nrThreads:
            while True:
                for t in threads:
                    if t.isAlive ():
                        t.join (1.0)
                    else:
                        del threads [threads.index (t)]
                if not threads:
                    break
            inform ("Builder threads finished.")


def generateTopLevelMakefile ():
    i = 0
    payload = "TEMPLATE = subdirs\n"
    for mod in manifest:
        if i == 0:
            payload = payload + "\nSUBDIRS +="
        payload = payload + " " + mod
        i = i + 1

    proName = Project + ".pro"
    # Generate the top-level Makefile.
    inform ("Creating %s and top level Makefile." % (proName))
    copyToFile (proName, payload)
    buildMakefile (proName)


def usage(rcode = 2):
#    print __doc__ % globals()
    print Project, Version
    print usageTxt % (Project, Version, "build.py", "build.py")
    sys.exit(rcode)


def checkThreading ():
    # check, if threading is desired.
    try:
        nrcpu = int(os.environ["SMP"])
    except:
        nrcpu = 1

    # check, if threading is available.
    if nrcpu > 1:
        try:
            import signal   # should improve signal distribution with threads
            from threading import BoundedSemaphore, Thread
        except:
            inform("Threaded build is not available")
        else:
            global nrThreads
            nrThreads = nrcpu


def main (argv):

# --------------Initialization----------------
    """
    Set up some basic features of the build environment
    """
    global progName
    progName = os.path.basename(argv[0])

    getOptions (argv)
    checkThreading ()
    initGlobals ()
    installChecks ()


# --------------Discovery---------------------
    """
    Acquire information about the existing environment
    (paths, compiler, qt features, etc)
    """
    global dis
    dis = Discovery (opts, usingSip, usingQt, usingPyQt, usingKDE, usingPyKDE, isPyKDE)
    dis.sanity (Release, opts)

    global proPatches, makefilePatches
    dis.setPatches (proPatches, makefilePatches)

    maindir = mkTempBuildDir ()
    compileChecks ()

    if usingQt or usingPyQt:
        # Generate the features file.
        generateFeatures(os.path.join (maindir, "features"))

    # We don't need the temporary build directory anymore.
    mkTempBuildDir(maindir)


# -------------Code Generation----------------
    """
    Generate code and makefiles
    """
    generateHook (dis)

    clibs   = []
    siplibs = []

    for mod in manifest:
        try:
            bldtype, postProc = modInfo [mod]
        except:
            print "\n*** WARNING: no module info for %s\n" % mod
            continue

        if bldtype == "clib":
            clibs.append (mod)
        elif bldtype == "siplib":
            siplibs.append (mod)

    for mod in clibs:
        generateCLibs (mod)
    print

    generateSipLibs (siplibs)
    generateTopLevelMakefile ()


if __name__ == "__main__":
    print
    try:
        main (sys.argv)
    except SystemExit:
        raise
    except:
        print internalError
        raise

