# Copyright (c) 2003  Jim Bublitz <jbublitz@nwinternet.com>

# This software is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this library; see the file COPYING.
# If not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys
import os
import os.path
import string
import re
import glob

False = 0
True  = 1

class DiscoverError (Exception):
    """
    The base class for all Discover Errors
    """
    def __init__ (self, msg = ""):
        self.msg = msg

    def __str__ (self):
        return self.msg


class Discover (object):
    def __init__ (self, target, searchPath):
        self.path       = ""
        self.ok         = False

        for searchEntry in searchPath:
            self.search (target, searchEntry)
            if self.ok: break

        if not self.ok:
            raise DiscoverError, "%s -- not found\n (%s, %s)" % (self.msgID, searchPath, target)
        else:
            self.inform ()


    def search (self, target, searchEntry):
        if not searchEntry:
            return

        if os.path.isdir (searchEntry)\
            and (not target or os.path.isfile (os.path.join (searchEntry, target))):
            self.path = searchEntry
            self.ok   = True

    def inform (self):
        print string.join ([self.msgID, "is", self.path])


class DiscoverPy (Discover):
    def __init__ (self, target = None, searchPath = [os.path.join (sys.prefix, "lib", "python")]):
        self.name    = "Py"
        self.vers    = ""
        self.versNum = 0
        self.major   = ""
        self.minor   = ""
        self.micro   = ""
        self.msgID   = "Python parent directory"

        Discover.__init__ (self, target, searchPath)


    def search (self, target, searchEntry):
        if not self.vers and not self.getVersion ():
            return

        if os.path.isdir (searchEntry + self.vers):
            self.path = searchEntry + self.vers
            self.ok   = True

    def getVersion (self):
        fullVers = string.split(sys.version)[0]

        vl = string.split(re.findall("[0-9.]*",fullVers)[0],".")
        self.major = vl[0]
        self.minor = vl[1]

        self.vers    = self.major + "." + self.minor
        self.versNum = int(self.major) * 10 + int(self.minor)

        return True

    def inform (self):
        print "Python version is %s" % self.vers
        print string.join ([self.msgID, "is", self.path])

class DiscoverPyIncl (Discover):
    def __init__ (self, target, searchPath):
        self.name  = "PyIncl"
        self.msgID = "Python include directory"
        Discover.__init__ (self, target, searchPath)

class DiscoverPyLib (Discover):
    def __init__ (self, target, searchPath):
        self.name  = "PyLib"
        self.msgID = "Python library directory"
        Discover.__init__ (self, target, searchPath)

class DiscoverQt3 (Discover):
    def __init__ (self, target, searchPath):
        self.name  = "Qt3"
        self.msgID = "Qt3 parent directory"
        Discover.__init__ (self, target, searchPath)

        self.vers    = ""
        self.versNum = 0
        self.major   = ""
        self.minor   = ""
        self.micro   = ""

    def getVersion (self, versFile):
        self.msgID = "finding Qt3 version"
        self.ok = False
        f = open (versFile)
        l = f.readline()

        while not self.ok and l:
            wl = string.split(l)
            if len(wl) == 3 and wl[0] == "#define":
                if wl[1] == "QT_VERSION":
                    qtv = wl[2]

                    if qtv[0:2] == "0x":
                        self.versNum = int(qtv,16)
                    else:
                        dec = int(qtv)
                        maj = dec / 100
                        min = (dec % 100) / 10
                        mic = (dec % 10)
                        self.versNum = (majj << 16) + (min << 8) + mic

                if wl[1] == "QT_VERSION_STR":
                    self.vers = wl[2][1:-1]
                    parts = string.split (self.vers, ".")
                    self.major = parts [0]
                    if len (parts) >= 2:
                        self.minor = parts [1]
                    if len (parts) >= 3:
                        self.micro = parts [2]

                if self.vers and self.versNum:
                    self.ok = True

            l = f.readline()

        f.close()
        if not self.ok:
            raise DiscoverError, "Qt3 version not found"
        else:
            print "Qt version is %s" % self.vers


    def getTag (self, tags, desc):
        """Convert a version number to a tag.

        self.vers is the version number.
        tags is the dictionary of tags keyed by version number.
        desc is the descriptive name of the package.

        Returns the corresponding tag.
        """
        tag = None

        vl = tags.keys()
        vl.sort()

        # For a snapshot use the latest tag.
        if self.versNum == 0:
            tag = tags[vl[-1]]
        else:
            for v in vl:
                if self.versNum < v:
                    tag = tags[v]
                    break

            if tag is None:
                raise DiscoverError, "invalid %s version: 0x%06x." % (desc, self.versNum)

        return tag

class DiscoverQt3Incl (Discover):
    def __init__ (self, target, searchPath):
        self.name  = "Qt3Incl"
        self.msgID = "Qt3 include directory (%s)" % target
        Discover.__init__ (self, target, searchPath)


class DiscoverQt3Lib (Discover):
    def __init__ (self, target, searchPath):
        self.name  = "Qt3Lib"
        self.msgID = "Qt3 lib directory (%s)" % target
        Discover.__init__ (self, target, searchPath)


class DiscoverKDE3 (Discover):
    def __init__ (self, target, searchPath):
        self.name  = "KDE3"
        self.msgID = "KDE3 parent directory"
        Discover.__init__ (self, target, searchPath)

        self.vers    = ""
        self.versNum = 0
        self.major   = ""
        self.minor   = ""
        self.micro   = ""

    def getVersion (self, versFile):
        if not os.path.isfile (versFile):
            return

        f = open (versFile)
        l = f.readline ()
        self.ok = False

        while not self.ok and l:
            wl = string.split(l)
            if len(wl) == 3 and wl[0] == "#define":
                if wl[1] == "KDE_VERSION_MAJOR":
                    self.major = string.strip (wl[2])

                if wl[1] == "KDE_VERSION_MINOR":
                    self.minor = string.strip (wl[2])

                if wl[1] == "KDE_VERSION_RELEASE":
                    self.micro = string.strip (wl[2])

            if self.major and self.minor and self.micro:
                self.ok = True

            l = f.readline()

        f.close()

        if self.ok:
            self.vers = string.join ([self.major, self.minor, self.micro], '.')
            self.versNum = int (self.major) << 16 + int (self.minor) << 8  + int (self.micro)
            print "KDE version is %s" % self.vers
        else:
            raise DiscoverError, "KDE version not found"


class DiscoverKDE3Incl (Discover):
    def __init__ (self, target, searchPath):
        self.name  = "KDE3Incl"
        self.msgID = "KDE3 include directory (%s)" % target
        Discover.__init__ (self, target, searchPath)


class DiscoverKDE3Lib (Discover):
    def __init__ (self, target, searchPath):
        self.name  = "KDE3Lib"
        self.msgID = "KDE3 library directory (%s)" % target
        Discover.__init__ (self, target, searchPath)


class DiscoverSipBin (Discover):
    def __init__ (self, target, searchPath):
        self.name  = "SipBin"
        self.bin   = target
        self.msgID = "sip code generator path (%s)" % (self.bin)
        Discover.__init__ (self, target, searchPath)

        self.bin   = os.path.join (self.path, self.bin)

        self.getVersion ()

    def getVersion (self):
        if os.system ("sip -V > sipv.tmp") or not os.access ("sipv.tmp", os.F_OK):
            raise DiscoverError, "trying to determine sip version"

        f = open ("sipv.tmp", "r")
        buff = f.read ()
        f.close ()
        os.remove ("sipv.tmp")

        if buff [3] == ".":
            self.vers = buff [0:5]
        else:
            self.vers = buff [0:3]

        print "sip version is %s" % self.vers

class DiscoverSipIncl (Discover):
    def __init__ (self, target, searchPath):
        self.name  = "SipIncl"
        self.msgID = "sip include directory (%s)" % target
        Discover.__init__ (self, target, searchPath)


class DiscoverSipLib (Discover):
    def __init__ (self, target, searchPath, isSipLib = True):
        if isSipLib:
            self.name = "SipLib"
            self.msgID = "sip lib directory (%s)" % target
        Discover.__init__ (self, target, searchPath)


    def search (self, target, searchEntry):
        if not searchEntry:
            return

        if os.path.isdir (searchEntry) and glob.glob(os.path.join (searchEntry, target)):
            self.path = searchEntry
            self.ok   = True

class DiscoverPyQtSip (Discover):
    def __init__ (self, target, searchPath):
        self.name   = "PyQtSip"
        self.msgID  = "PyQt sip files directory"
        self.target = target
        Discover.__init__ (self, target, searchPath)

        self.getVersion ()

    def inform (self):
        self.path = os.path.dirname (os.path.join (self.path, self.target))
        Discover.inform (self)

    def getVersion (self):
        f = open (os.path.join (self.path, "qtmod.sip"), "r")
        for line in f.readlines ():
            if string.find (line, "static unsigned PYQT_VERSION") >= 0:
                self.versNum  = string.strip (string.split (line, "=") [1])
                break
        else:
            raise DiscoverError, "PyQt sip version not found"

        if self.versNum [2] != "0":
            self.versMum = string.replace (self.versNum, "0x", "0x0")
        self.major = str (int (self.versNum [2:4]))
        self.minor = str (int (self.versNum [4:6]))
        self.micro = str (int (self.versNum [6:8]))

        if self.micro == 0:
            self.vers = "%s.%s" % (self.major, self.minor)
        else:
            self.vers = "%s.%s.%s" % (self.major, self.minor, self.micro)

        print "PyQt sip version is %s" % self.vers

class DiscoverPyQtLib (DiscoverSipLib):
    def __init__ (self, target, searchPath):
        self.name  = "PyQtLib"
        self.msgID = "PyQt lib directory"
        DiscoverSipLib.__init__ (self, target, searchPath, False)

        self.getVersion ()

    def getVersion (self):
        try:
            from qt import PYQT_VERSION, PYQT_VERSION_STR
        except:
            raise DiscoverError, "Couldn't import qt module from PyQt"

        self.vers    = PYQT_VERSION_STR
        self.versNum = PYQT_VERSION

        print "PyQt lib version is %s" % self.vers


class DiscoverPyKDESip (Discover):
    def __init__ (self, target, searchPath):
        self.name   = "PyKDESip"
        self.msgID  = "PyKDE sip files directory"
        self.target = target
        Discover.__init__ (self, target, searchPath)

        self.getVersion ()

    def inform (self):
        self.path = os.path.dirname (os.path.join (self.path, self.target))
        Discover.inform (self)

    def getVersion (self):
        f = open (os.path.join (self.path, "qtmod.sip"), "r")
        for line in f.readlines ():
            if string.find (line, "static unsigned PYQT_VERSION") >= 0:
                self.versNum  = string.strip (string.split (line, "=") [1])
                break
        else:
            raise DiscoverError, "PyQt sip version not found"

        if self.versNum [2] != "0":
            self.versMum = string.replace (self.versNum, "0x", "0x0")
        self.major = str (int (self.versNum [2:4]))
        self.minor = str (int (self.versNum [4:6]))
        self.micro = str (int (self.versNum [6:8]))

        if self.micro == 0:
            self.vers = "%s.%s" % (self.major, self.minor)
        else:
            self.vers = "%s.%s.%s" % (self.major, self.minor, self.micro)

        print "PyQt sip version is %s" % self.vers

class DiscoverPyKDELib (DiscoverSipLib):
    def __init__ (self, target, searchPath):
        self.name  = "PyKDELib"
        self.msgID = "PyKDE lib directory"
        DiscoverSipLib.__init__ (self, target, searchPath, False)

        self.getVersion ()

    def getVersion (self):
        try:
            from kdecore import PyKDE
        except:
            raise DiscoverError, "Couldn't import kde module from PyQt"

        self.vers    = PyKDE.versionString ()
        self.major   = PyKDE.major ()
        self.minor   = PyKDE.minor ()
        self.micro   = PyKDE.release ()
        self.versNum = (int (self.major) << 16) + (int (self.minor) << 8) + int (self.micro)

        print "PyKDE lib version is %s" % self.vers

class DiscoverQMake (Discover):
    def __init__ (self, target, searchPath):
        self.name  = "QMake"
        self.msgID = "Makefile generator path (%s)" % target
        Discover.__init__ (self, target, searchPath)

class DiscoverInstall (Discover):
    def __init__ (self, target, searchPath):
        self.name  = "Install"
        self.msgID = "Install directory"
        try:
            Discover.__init__ (self, target, searchPath)
        except:
            try:
                os.mkdir (searchPath)
            except:
                raise DiscoverError, "Unable to create install directory %s" % searchPath
            Discover.__init__ (self, target, searchPath)


class Discovery (object):
    def __init__ (self, opts, usingSip = False, usingQt = False, usingPyQt = False,\
                  usingKDE = False, usingPyKDE = False, isPyKDE = False):
        try:
            self.discoverPy (opts)
        except DiscoverError, value:
            self.error (value.msg)

        if isPyKDE:
            try:
                self.discoverHome (opts)
            except DiscoverError, value:
                self.error (value.msg)
        else:
            self.home = ""

        if usingSip or usingPyQt:
            try:
                self.discoverSip (opts)
            except DiscoverError, value:
                self.error (value.msg)
        else:
            self.sipBin  = None
            self.sipLib  = None
            self.sipIncl = None

        if usingQt or usingPyQt:
            try:
                self.discoverQt3 (opts)
            except DiscoverError, value:
                self.error (value.msg)
        else:
            self.qt        = None
            self.qtLib     = None
            self.qtIncl    = None

        if usingPyQt:
            try:
                self.discoverPyQt (opts)
            except DiscoverError, value:
                self.error (value.msg)
        else:
            self.pyQtSip = None
            self.pyQtLib = None

        if usingKDE:
            try:
                self.discoverKDE3 (opts)
            except DiscoverError, value:
                self.error (value.msg)
        else:
            self.kde       = None
            self.kdeLib    = None
            self.kdeIncl   = None

        if usingPyKDE:
            try:
                self.discoverPyKDE (opts)
            except DiscoverError, value:
                self.error (value.msg)
        else:
            self.pyKDESip = None
            self.pyKDELib = None

        self.discoverMake (opts)


    def error (self, errmsg):
        print "\n*** Error: %s\n" % errmsg
        print "Please review the installation documentation before reporting errors"
        print "When reporting errors, include the complete output of this program\n"
        sys.exit (1)

    def discoverPy (self, opts):
        """
        -t dir         the directory containing the Python library directories
        """
        self.py     = DiscoverPy ()
        if opts.has_key ("-t"):
            pyLibDir = [opts ["-t"]]
        else:
            pyLibDir = [os.path.join (self.py.path, "config")]
        self.pyLib  = DiscoverPyLib (string.join (["libpython", self.py.vers, ".a"], ""), pyLibDir)
        self.pyIncl = DiscoverPyIncl ("Python.h", [os.path.join (sys.prefix, "include", "python" + self.py.vers)])
        print

    def discoverHome (self, opts):
        self.home = os.getenv ("HOME")
        if not self.home:
            self.home = pwd.getpwdnam (os.getlogin ()) [5]

        if self.home and not os.path.isdir (self.home):
            self.home = ""

        if not self.home:
            raise DiscoverError, "Current user's $HOME path not found"
        else:
            print "Home directory is %s\n" % self.home


    def discoverSip (self, opts):
        """
        -e dir         the directory containing the SIP header files
        -p dir         the name of the SIP code generator [default sip]
        -s dir         the directory containing the SIP module
        """
        if opts.has_key ("-p"):
            bin = opts ["-p"]
        else:
            bin = "sip"
        sipBinSearchPath = ["/usr/local/bin", "/usr/bin"]
        self.sipBin = DiscoverSipBin (bin, sipBinSearchPath)

        if opts.has_key ("-s"):
            sipLibDir = [opts ["-s"]]
        else:
            sipLibDir = [os.path.join (self.py.path, "site-packages")]
        self.sipLib = DiscoverSipLib ("libsip*", sipLibDir)

        if opts.has_key ("-e"):
            sipInclDir = [opts ["-e"]]
        else:
            sipInclDir = [self.pyIncl.path]
        self.sipIncl = DiscoverSipIncl ("sipQt.h",  sipInclDir)
        print

    def discoverQt3 (self, opts):
        """
        -q dir         the Qt base directory
        -i dir         the directory containing the Qt header files
        -r dir         the directory containing the Qt library
        """
        if opts.has_key ("-q"):
            qtSearchPath = [opts ["-q"]]
        else:
            qtSearchPath = []
            try:
                qtSearchPath.append (os.environ ["QTDIR"])
            except:
                pass
            qtSearchPath.append (os.path.join ("/usr", "lib", "qt3"))
        self.qt = DiscoverQt3 (None, qtSearchPath)

        if opts.has_key ("-r"):
            qtLibDir = [opts ["-r"]]
        else:
            qtLibDir = [os.path.join (self.qt.path, "lib")]
        self.qtLib = DiscoverQt3Lib ("libqt-mt.so.3", qtLibDir)

        if opts.has_key ("-i"):
            qtInclDir = [opts ["-i"]]
        else:
            qtInclDir = [os.path.join (self.qt.path, "lib")]
        self.qtIncl = DiscoverQt3Incl ("qglobal.h", [os.path.join (self.qt.path, "include"), qtInclDir])

        self.qt.getVersion (os.path.join (self.qtIncl.path, "qglobal.h"))
        print

    def discoverPyQt (self, opts):
        """
        -v dir         the directory containing the PyQt sip files
        """
        self.pyQtLib = DiscoverPyQtLib ("libqtcmodule*", [self.sipLib.path])

        if opts.has_key ("-v"):
            pyQtSipPath = [opts ["-v"]]
        else:
            pyQtSipPath = ["../", "/usr/local/", "/usr/local/src/", "/usr/src/", "/usr/share/sip", "/usr/local/share/sip",\
                           "/usr/share/sip/qt"]
        self.pyQtSip = DiscoverPyQtSip (os.path.join ("PyQt-x11-gpl-" + self.pyQtLib.vers, "sip", "qtmod.sip"), pyQtSipPath)
        print

    def discoverKDE3 (self, opts):
        """
        -k dir         the KDE base directory
        -n dir         the directory containing the KDE header files
        -o dir         the directory containing the KDE lib files
        """

        if opts.has_key ("-k"):
            kdeSearchPath = [opts ["-k"]]
        else:
            kdeSearchPath = []
            try:
                kdeSearchPath.append (os.environ ["KDEDIR"])
            except:
                pass
            kdeSearchPath.append (os.path.join ("/opt", "kde3"))
            kdeSearchPath.append ("/usr")
        self.kde = DiscoverKDE3 (None, kdeSearchPath)

        if opts.has_key ("-o"):
            kdeLibDir = [opts ["-o"]]
        else:
            kdeLibDir = [os.path.join (self.kde.path, "lib")]
        self.kdeLib = DiscoverKDE3Lib ("libkdecore.so", kdeLibDir)

        if opts.has_key ("-n"):
            kdeInclDir = [opts ["-n"]]
        else:
            kdeInclDir = [os.path.join (self.kde.path, "include")]
            kdeInclDir.append (os.path.join (self.kde.path, "include", "kde")) # Red Hat
        self.kdeIncl = DiscoverKDE3Incl ("kapplication.h", kdeInclDir)
        self.kde.getVersion (os.path.join (self.kdeIncl.path, "kdeversion.h"))
        print

    def discoverPyKDE (self, opts):
        self.pyKDELib = DiscoverPyKDELib ("libkdecorecmodule*", [self.sipLib.path])

        pyKDESipPath = ["../", "/usr/local/", "/usr/local/src/", "/usr/src/", "/usr/share/sip", "/usr/local/share/sip",\
                           "/usr/share/sip/kde"]
        self.pyKDESip = DiscoverPyQtSip (os.path.join ("PyKDE-" + self.pyKDELib.release, "sip", "kapplication.sip"), pyKDESipPath)
        print

    def discoverMake (self, opts):
        """
        -d dir         where PyKDE will be installed
        -g prog        the complete path/name of the Makefile generator"
        -m prog        the name of the Make program
        """
        if opts.has_key ("-m"):
            self.make = opts ["-m"]
        else:
            self.make = "make"
        print "make program is %s" % self.make

        if opts.has_key ("-g"):
            self.makeGen     = os.path.basename (opts ["-g"])
            self.makeGenPath = [os.path.dirname (opts ["-g"])]
        else:
            if self.qt is None:
                if opts.has_key ("-q"):
                    qtSearchPath = [opts ["-q"]]
                else:
                    qtSearchPath = []
                    try:
                        qtSearchPath.append (os.environ ["QTDIR"])
                    except:
                        pass
                    qtSearchPath.append (os.path.join ("/usr", "lib", "qt3"))
                qtpath = DiscoverQt3 (None, qtSearchPath)
            else:
                qtpath = self.qt.path
            self.makeGen     = "qmake"
            self.makeGenPath = [os.path.join (qtpath, "bin")]

        self.qmakePath = DiscoverQMake (self.makeGen, self.makeGenPath)
        self.qmake = os.path.join (self.qmakePath.path, self.makeGen)

        if opts.has_key ("-d"):
            instDir = [opts ["-d"]]
        else:
           instDir = [os.path.join (self.py.path, "site-packages")]
        self.installDir = DiscoverInstall (None, instDir)
        print

    def sanity (self, release, opts):
        if opts.has_key ("-u"):
            vCheck = opts ["-u"]
        else:
            vCheck = "all"

        msg = ""

        base = release [0:3]
        if self.sipBin and self.sipBin.vers [0:3] != base:
            msg = "sip version (%s) not compatible with %s version (%s)"
            if vCheck == "all":
                raise DiscoverError, msg % (self.sipBin.vers, "this", base)
            elif self.pyQtSip and self.sipBin [0:3] != self.pyQtSip.vers [0:3]:
                raise DiscoveryError, msg % (self.sipBin.vers, "PyQt sip files", self.pyQtSip.vers [0:3])
            elif self.pyQtLib and self.sipBin [0:3] != self.pyQtLib.vers [0:3]:
                raise DiscoveryError, msg % (self.sipBin.vers, "PyQt libs", self.pyQtSip.vers [0:3])
            else:
                print "%s -- ignored" % (msg % (self.sipBin.vers, "this", base))

        if self.pyQtSip and self.pyQtSip.vers [0:3] != base:
            msg = "PyQt sip files version (%s) not compatible with %s version (%s)"
            if vCheck in ["all", "nosip"]:
                raise DiscoverError, msg % (self.pyQtSip.vers, "this", base)
            elif self.pyQtLib and self.pyQtSip.vers [0:3] != pyQtLib.vers  [0:3]:
                raise DiscoverError, msg % (self.pyQtSip.vers, "PyQt libs", self.pyQtLib.vers [0:3])
            else:
                print "%s -- ignored" % (msg % (self.pyQtSip.vers, "this", self.pyQtLib.vers [0:3]))

        if self.pyQtLib and self.pyQtLib.vers [0:3] != base:
            msg = "PyQt sip files version (%s) not compatible with this version (%s)" % (self.pyQtLib.vers, base)
            if vCheck in ["all", "nosip"]:
                raise DiscoverError, msg
            else:
                print "%s -- ignored" % msg

        if msg:
            print "/n*** WARNING *** possibile version mismatch ignored\n"
        else:
            print "Component versions are compatible (%s)" % base

    def setPatches (self, proPatches, makefilePatches):
        # Set the list of directories containing header files.
        incdirs = ". " + self.pyIncl.path

        if self.sipIncl and (self.sipIncl.path != self.pyIncl.path):
            incdirs = incdirs + " " + self.sipIncl.path

        if self.qt:
            incdirs = incdirs + " " + self.qtIncl.path
            makefilePatches ["QTDIR_LIB"]    = [re.compile ("(^CC)",re.M), "QTDIR    = " + self.qt.path + "\n\\1"]
            makefilePatches ["QTDIR_SUBDIR"] = [re.compile ("(^MAKEFILE)",re.M), "QTDIR    = " + self.qt.path + "\n\\1"]
            proPatches ["THREAD"] = [re.compile ("@BL_THREAD@",re.M), "thread"]
            print "Qt thread support is enabled."

        if self.kde:
            incdirs = incdirs + " " + self.kdeIncl.path
            proPatches ["KDE_INCLUDES"] = [re.compile("@KDE_INCLUDEPATH@",re.M), self.kdeIncl.path]
            proPatches ["KDELIBS"]      = [re.compile("@BL_KDEDIR@",re.M), self.kdeLib.path]
            makefilePatches ["KDEDIR_LIB"]    = [re.compile ("(^CC)",re.M), "KDEDIR    = " + self.kde.path + "\n\\1"]
            makefilePatches ["KDEDIR_SUBDIR"] = [re.compile ("(^MAKEFILE)",re.M), "KDEDIR    = " + self.kde.path + "\n\\1"]

        proPatches ["INCLUDEPATH"]  = [re.compile ("@BL_INCLUDEPATH@",re.M), incdirs]

        ld = os.path.join ("..", "libs")

        sitePkgs = os.path.join (self.py.path, "site-packages")
        if self.pyQtLib.path == sitePkgs:
            linkPath = "%s -L %s" % (ld, sitePkgs)
        else:
            linkPath = "%s -L %s -L%s" % (ld, self.pyqtLib.path, sitePkgs)

        proPatches ["LIBS"] = [re.compile ("@BL_LINKDIRS@",re.M), linkPath]

        makefilePatches ["MAKE_LIB"]    = [re.compile ("(^CC)",re.M), "MAKE     = \"" + self.make + "\"\n\\1"]
        makefilePatches ["MAKE_SUBDIR"] = [re.compile ("(^MAKEFILE)",re.M), "MAKE     = \"" + self.make + "\"\n\\1"]
        makefilePatches ["QMAKE1"]      = [re.compile ("(^QMAKE[ \t]*=).*$",re.M), "\\1 \"" + self.qmake + "\""]
        makefilePatches ["QMAKE2"]      = [re.compile (" qmake_all",re.M), ""]

        print


