"""
autotest.py - performs simple tests on some %MemberCode
and %MappedTypes from PyKDE. The primary purpose of the
testing in this case is to verify that handwritten code
is correct and functional.

Some methods don't lend themselves to automated testing
and are tested functionally (and usually interactively)
in other places (see the examples3/ subdirectory). Methods
from the kjs and khtml modules aren't tested here because
the setup details are more complex. Hopefully these will
have tests in the near future.

The testing in this module is geared to PyKDE for KDE >= 3,
as support for KDE 2 will be diminishing in the near future.

Some of the tests in this module are locale dependent
(dates, currency, etc) and may need to be modified to
work in your locale.
"""

"""
Copyright 2003 Jim Bublitz

Terms and Conditions

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
COPYRIGHT HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

Except as contained in this notice, the name of the copyright holder shall
not be used in advertising or otherwise to promote the sale, use or other
dealings in this Software without prior written authorization from the
copyright holder.
"""

import sys, os, signal, unittest

from kdecore import KApplication, KCmdLineArgs, KAboutData, KStandardDirs, KGlobal, KProcIO, KProcess, KURL,\
                    KEntry, KEntryKey, testKEntryMap
from kdeui import KMainWindow, KColor, KCommandHistory, KDateValidator, KIntValidator, KFloatValidator,\
                  KEdit, KEditListBox, KStringListValidator, KMimeTypeValidator, KStdAction, testKActionList
from kio import KIO, KMimeType, testKIOMetaData, testKIOUDSEntry, testKIOUDSEntryList
from kfile import KFile

from qt import QString, QValidator, QDir, QPushButton, SIGNAL, QStringList

False = 0
True  = 1

# ------------------- auxiliary classes ----------------------------------

class MainWin (KMainWindow):
    def __init__ (self, *args):
        apply (KMainWindow.__init__, (self,) + args)

        self.setGeometry (10, 10, 400, 400)

        self.edit = KEdit (self)
        self.edit.setGeometry (10, 10, 220, 150)

        textList = ["Now is the winter of our discontent",\
                    "made glorious summer by this sun of York;",\
                    "and all the clouds that lour'd upon our house",\
                    "in the deep bosom of the ocean buried."]

        for line in textList:
            self.edit.insertLine (line)

        self.editListBox = KEditListBox (self)
        self.editListBox.setGeometry (10, 180, 220, 200)

        self.testBtn = QPushButton ("test", self)
        self.testBtn.setGeometry (290, 355, 60, 22)
        self.connect (self.testBtn, SIGNAL ("clicked ()"), self.doTest)

        self.newAction    = KStdAction.openNew (self.noSlot, self.actionCollection (), "openNew")
        self.openAction   = KStdAction.open (self.noSlot, self.actionCollection (), "open")
        self.saveAction   = KStdAction.save (self.noSlot, self.actionCollection (), "save")
        self.saveAsAction = KStdAction.saveAs (self.noSlot, self.actionCollection (), "saveAs")
        self.printAction  = KStdAction.print_ (self.noSlot, self.actionCollection (), "print")
        self.quitAction   = KStdAction.quit (self.noSlot, self.actionCollection (), "quit")

    def noSlot (self):
        pass

    def doTest (self):
        suite = unittest.TestSuite ()
        suite.addTest (ULongLongTestCase ('runTest'))
        suite.addTest (DCOPClientTestCase ("runTest"))
        suite.addTest (KApplicationTestCase ("runTest"))
        suite.addTest (KCharsetsTestCase ("runTest"))
        suite.addTest (KColorTestCase ("runTest"))
        suite.addTest (KCommandHistoryTestCase ("runTest"))
        suite.addTest (KDateValidatorTestCase ("runTest"))
        suite.addTest (KMimeTypeValidatorTestCase ("runTest"))
        suite.addTest (KStringListValidatorTestCase ("runTest"))
        suite.addTest (KIntValidatorTestCase ("runTest"))
        suite.addTest (KFloatValidatorTestCase ("runTest"))
        suite.addTest (KEditTestCase ("runTest"))
        suite.addTest (KEditListBoxTestCase ("runTest"))
        suite.addTest (KLocaleTestCase ("runTest"))
        suite.addTest (KFileTestCase ("runTest"))
        suite.addTest (KMimeTypeTestCase ("runTest"))
        suite.addTest (KProcIOTestCase ("runTest"))
        suite.addTest (KURLListTestCase ("runTest"))
        suite.addTest (KIOSlaveTestCase ("runTest"))
        suite.addTest (KIONetRCAutoLoginTestCase ("runTest"))
        suite.addTest (KIOglobalTestCase ("runTest"))
        suite.addTest (KActionCollectionTestCase ("runTest"))
        suite.addTest (KActionListTestCase ("runTest"))
        suite.addTest (KEntryMapTestCase ("runTest"))
        runner = unittest.TextTestRunner(sys.stdout, verbosity=2)
        print
        print
        print "testing ..."
        print
        result = runner.run(suite)
        print
        print

        sys.exit (0)

# ------------------- test case classes ----------------------------------

class BasePyKDETestCase (unittest.TestCase):
    pass
#    def setUp (self):
#        description = "A basic application template"
#        version     = "1.0"
#        aboutData   = KAboutData ("", "",\
#            version, description, KAboutData.License_GPL,\
#            "(C) 2003 whoever the author is")

#        aboutData.addAuthor ("author1", "whatever they did", "email@somedomain")
#        aboutData.addAuthor ("author2", "they did something else", "another@email.address")

#        KCmdLineArgs.init (sys.argv, aboutData)

#        KCmdLineArgs.addCmdLineOptions ([("+files", "File to open")])

#        self.app = KApplication ()
#        self.mainWindow = MainWin (None, "main window")
#        mainWindow.show()
#        app.exec_loop()

class ULongLongTestCase (BasePyKDETestCase):
    def runTest (self):
        """
        Test some simple conversions from unsigned long long to QString
        using %MappedType ulonglong
        """
        from kio import KIO
        self.assert_ (KIO.number (0).compare (QString ("0")) == 0, "number (0) failed")
        self.assert_ (KIO.convertSize (0).compare (QString ("0 B")) == 0, "convertSize (0) failed")
        self.assert_ (KIO.number (1).compare (QString ("1")) == 0, "number (1) failed")
        self.assert_ (KIO.convertSize (1).compare (QString ("1 B")) == 0, "convertSize (1) failed")
        self.assert_ (KIO.number (1000).compare (QString ("1000")) == 0, "number (1000) failed")
        self.assert_ (KIO.convertSize (1000).compare (QString ("1,000 B")) == 0, "convertSize (1000) failed")
        self.assert_ (KIO.number (2048).compare (QString ("2048")) == 0, "number (2048) failed")
        self.assert_ (KIO.convertSize (2048).compare (QString ("2.0 KB")) == 0, "convertSize (2048) failed")
        self.assert_ (KIO.number (1234567890123456789).compare (QString ("1234567890123456789")) == 0,\
                      "number (1234567890123456789) failed")
        self.assert_ (KIO.convertSize (1234567890123456789).compare (QString ("1,122,833.0 TB")) == 0,\
                      "convertSize (1234567890123456789) failed")


class DCOPClientTestCase (BasePyKDETestCase):
    def runTest (self):
        """
        Tests both member code (remoteObjects, remoteInterfaces, remoteFunctions)
        but also tests mapped type QCStringList (all lists in this test case
        are QCStringLists)

        Uses KApplication's dcopClient, so doesn't need to import dcop module
        directly
        """

        d = KApplication.dcopClient ()
        self.assert_ (d.isRegistered () == 1)
        regApps = d.registeredApplications ()
        self.assert_ (len (regApps), "registerApplications -- empty list")
        self.assert_ (self.checkListForValue (regApps, "klauncher"), "registerApplciations -- no klauncher")

        remObjs, ok = d.remoteObjects ("klauncher")
        self.assert_ (ok, "remoteObjects returned not ok")
        self.assert_ (len (remObjs), "remoteObjects -- empty list")
        self.assert_ (self.checkListForValue (remObjs, "klauncher"), "remoteObjects -- no klauncher")

        ifs, ok = d.remoteInterfaces ("klauncher", "klauncher")
        self.assert_ (ok, "remoteInterfaces returned not ok")
        self.assert_ (len (ifs), "remoteInterfaces -- empty list")
        self.assert_ (self.checkListForValue (ifs, "DCOPObject"), "remoteInterfaces -- no DCOPObject")

        fcns, ok = d.remoteFunctions ("klauncher", "klauncher")
        self.assert_ (ok, "remoteFunctions returned not ok")
        self.assert_ (len (fcns), "remoteFunctions -- empty list")
        self.assert_ (self.checkListForValue (fcns, "QCStringList interfaces()"), "remoteFunctions -- no QCStringList interfaces()")


    def checkListForValue (self, list, value):
        flag = False
        for item in list:
           flag = str (item) == value
           if flag: break

        return flag


class KApplicationTestCase (BasePyKDETestCase):
    def runTest (self):
        result, error, dcopService, pid = KApplication.startServiceByName ("KWrite", QString.null, "kwrite")
        if pid: os.kill (pid, signal.SIGKILL)
        self.assert_ (result == 0, "startServiceByName returned an error code")
        self.assert_ (str(error) == "", "startServiceByName returned an error string")
        self.assert_ (str(dcopService) == "kwrite-" + str (pid), "startServiceByName returns incorrect dcopService")
        self.assert_ (pid, "startServiceByName returned pid == 0")

        result, error, dcopService, pid = KApplication.startServiceByDesktopName ("kwrite", QString.null, "kwrite")
        if pid: os.kill (pid, signal.SIGKILL)
        self.assert_ (result == 0, "startServiceByDesktopName returned an error code")
        self.assert_ (str(error) == "", "startServiceByDesktopName returned an error string")
        self.assert_ (str(dcopService) == "kwrite-" + str (pid), "startServiceByDesktopName returns incorrect dcopService")
        self.assert_ (pid, "startServiceByDesktopName returned pid == 0")

        stddir = KStandardDirs ()
        kwPath = stddir.findResource ("apps", "Editors/kwrite.desktop")
        result, error, dcopService, pid = KApplication.startServiceByDesktopPath (kwPath, QString.null, "kwrite")
        if pid: os.kill (pid, signal.SIGKILL)
        self.assert_ (result == 0, "startServiceByDesktopPath returned an error code")
        self.assert_ (str(error) == "", "startServiceByDesktopPath returned an error string")
        self.assert_ (str(dcopService) == "kwrite-" + str (pid), "startServiceByDesktopPath returns incorrect dcopService")
        self.assert_ (pid, "startServiceByDesktopPath returned pid == 0")

        result, error, pid = KApplication.kdeinitExec ('kwrite')
        if pid: os.kill (pid, signal.SIGKILL)
        self.assert_ (result == 0, "kdeinitExec returned an error code")
        self.assert_ (str(error) == "", "kdeinitExec returned an error string")
        self.assert_ (pid, "kdeinitExec returned pid == 0")


class KCharsetsTestCase (BasePyKDETestCase):
    def runTest (self):
        cs = KGlobal.charsets ()
        tc, ok = cs.codecForName2 ("Big5")
        self.assert_ (ok, "codecForName2 returned not ok")
        self.assert_ (tc, "codecForName2 returned null")
        self.assert_ (str (tc.name ()) == "Big5", "codecForName2 returned incorrect codec")


class KColorTestCase (BasePyKDETestCase):
    def runTest (self):
        crgb = KColor ()
        r    = 64
        g    = 128
        b    = 255
        crgb.setRgb (r, g, b)
        r1, g1, b1 = crgb.rgb ()

        chsv = KColor ()
        h    = 10
        s    = 20
        v    = 30
        chsv.setHsv (h, s, v)

        h1, s1, v1 = chsv.hsv ()

        self.assert_ (r == r1, "rgb returned incorrect r value")
        self.assert_ (g == g1, "rgb returned incorrect g value")
        self.assert_ (b == b1, "rgb returned incorrect b value")
        self.assert_ (h == h1, "hsv returned incorrect h value")
        self.assert_ (s == s1, "hsv returned incorrect s value")
        self.assert_ (v == v1, "hsv returned incorrect v value")


class KCommandHistoryTestCase (BasePyKDETestCase):
    def runTest (self):
        ch = KCommandHistory ()
        self.assert_ (ch.undoLimit (), "null undo limit")
        self.assert_ (ch.redoLimit (), "null redo limit")

        ch.setUndoLimit (30)
        ch.setRedoLimit (10)
        self.assert_ (ch.undoLimit () == 30, "incorrect undo limit")
        self.assert_ (ch.redoLimit () == 10, "incorrect redo limit")


class KDateValidatorTestCase (BasePyKDETestCase):
    def runTest (self):
        dv = KDateValidator ()
        state, pos = dv.validate ("1-4-03")
        self.assert_ (state == QValidator.Acceptable, "date validator error")
        state, pos = dv.validate ("1-")
        self.assert_ (state == QValidator.Intermediate, "date validator error")

class KMimeTypeValidatorTestCase (BasePyKDETestCase):
    def runTest (self):
        mv = KMimeTypeValidator (mainWindow)
        state, pos = mv.validate ("text/plain")
        self.assert_ (state == 2)
        state, pos = mv.validate ("/text/plain")
        self.assert_ (state == 0)

#----------------- KStringListValidator -------------------------

class KStringListValidatorTestCase (BasePyKDETestCase):
    def runTest (self):
        sv = KStringListValidator ()
        sl = QStringList ()
        sl.append ("Hello")
        sl.append ("Goodbye")
        sv.setStringList (sl)

        sv.setRejecting (False)
        state, pos = sv.validate ("Hello")
        self.assert_ (state == 2)
        state, pos = sv.validate ("Good")
        self.assert_ (state == 1)
        state, pos = sv.validate ("Something")
        self.assert_ (state == 0)

        sv.setRejecting (True)
        state, pos = sv.validate ("Hello")
        self.assert_ (state == 1)
        state, pos = sv.validate ("Good")
        self.assert_ (state == 2)
        state, pos = sv.validate ("Something")
        self.assert_ (state == 2)

class KIntValidatorTestCase (BasePyKDETestCase):
    def runTest (self):
        iv = KIntValidator (mainWindow)
        iv.setRange (10, 40)
        state, pos = iv.validate ("3")
        self.assert_ (state == QValidator.Intermediate, "int validator error")
        state, pos = iv.validate ("33")
        self.assert_ (state == QValidator.Acceptable, "int validator error")
        state, pos = iv.validate ("33z")
        self.assert_ (state == QValidator.Invalid, "int validator error")


class KFloatValidatorTestCase (BasePyKDETestCase):
    def runTest (self):
        fv = KFloatValidator (mainWindow)
        fv.setRange (10.0, 40.0)
        state, pos = fv.validate ("3")
        self.assert_ (state == QValidator.Invalid, "float validator error")
        state, pos = fv.validate ("3.3")
        self.assert_ (state == QValidator.Invalid, "float validator error")
        state, pos = fv.validate ("33.3")
        self.assert_ (state == QValidator.Acceptable, "float validator error")
        state, pos = fv.validate ("33z")
        self.assert_ (state == QValidator.Invalid, "float validator error")


class KEditTestCase (BasePyKDETestCase):
    def runTest (self):
        row, col = mainWindow.edit.posToRowCol (50)
        self.assert_ (row == 1, "incorrect row")
        self.assert_ (col == 14, "incorrect col")


class KEditListBoxTestCase (BasePyKDETestCase):
    def runTest (self):
        itemList = ["one", "two", "three", "four"]
        mainWindow.editListBox.insertStrList (itemList, -1)
        retrList = mainWindow.editListBox.items ()
        self.assert_ (len (retrList) == len (itemList), "item list wrong size")
        for i in range (len (retrList)):
            self.assert_ (str(retrList [i]) == itemList [i], "incorrect item value")

class KLocaleTestCase (BasePyKDETestCase):
    def runTest (self):
        locale = KGlobal.locale ()
        rm, ok = locale.readMoney ("$12.34")
        self.assert_ (ok, "readMoney returned not ok")
        self.assert_ (rm == 12.34, "readMoney returned bad value")

        rn, ok = locale.readNumber ("1,234,567")
        self.assert_ (ok, "readNumber returned not ok")
        self.assert_ (rn == 1234567.0, "readNumber returned bad value")

        qd, ok = locale.readDate (QString ("2003-10-10"))
        self.assert_ (qd.isValid (), "QDate is invalid")
        self.assert_ (ok, "readDate returned not ok")
        self.assert_ (qd.day () == 10, "wrong day")
        self.assert_ (qd.month (), "wring month")
        self.assert_ (qd.year (), "wrong year")

        qtm, ok = locale.readTime ("11:12:13")
        self.assert_ (ok, "readTime returned not ok")
        self.assert_ (qtm.isValid (), "QTime is invalid")
        self.assert_ (qtm.hour () == 11, "wrong hour")
        self.assert_ (qtm.minute () == 12, "wrong minute")
        self.assert_ (qtm.second () == 13, "wrong second")


class KFileTestCase (BasePyKDETestCase):
    def runTest (self):
        # true cases
        self.assert_ (KFile.isSortByName (QDir.Name))
        self.assert_ (KFile.isSortBySize (QDir.Size))
        self.assert_ (KFile.isSortByDate (QDir.Time))
        self.assert_ (KFile.isSortDirsFirst (QDir.DirsFirst))
        self.assert_ (KFile.isSortCaseInsensitive (QDir.IgnoreCase))
        self.assert_ (KFile.isDefaultView (KFile.Default))
        self.assert_ (KFile.isSimpleView (KFile.Simple))
        self.assert_ (KFile.isDetailView (KFile.Detail))
        self.assert_ (KFile.isSeparateDirs (KFile.SeparateDirs))
        self.assert_ (KFile.isPreviewContents (KFile.PreviewContents))

        # false cases
        self.assert_ (not KFile.isSortByName (QDir.Size))
        self.assert_ (not KFile.isSortBySize (QDir.Name))
        self.assert_ (not KFile.isSortByDate (QDir.Size))
        self.assert_ (not KFile.isSortDirsFirst (QDir.Name))
        self.assert_ (not KFile.isSortCaseInsensitive (QDir.Name))
        self.assert_ (not KFile.isSimpleView (KFile.Default))
        self.assert_ (not KFile.isDetailView (KFile.Simple))
        self.assert_ (not KFile.isSeparateDirs (KFile.Detail))
        self.assert_ (not KFile.isPreviewContents (KFile.SeparateDirs))



class KMimeTypeTestCase (BasePyKDETestCase):
    def runTest (self):
        m    = open ('autotest.py', 'r')
        buff = m.read ()
        m.close ()
        ptr, acc = KMimeType.findByContent (buff)
        self.assert_ (acc == 90, "expected accuracy == 90")
        self.assert_ (str (ptr.name ()) == 'text/plain', "wrong MIME type: %s" % ptr)
        ptr, acc = KMimeType.findByFileContent ('autotest.py')
        self.assert_ (acc == 90, "expected accuracy == 90")
        self.assert_ (str (ptr.name ()) == 'text/plain', "wrong MIME type: %s" % ptr)



class KProcIOTestCase (BasePyKDETestCase):
    def runTest (self):
        p = KProcIO ()
        p.setExecutable ("python")
        p.setArguments (["testKProcIO.py"])
        p.connect (p, SIGNAL ("readReady (KProcIO *)"), self.procRead)
        p.enableReadSignals (1)
        p.start (KProcess.Block)

    def procRead (self, p):
        result, buff, partial = p.readln ()
        if result != -1:
            self.assert_ (result == 5, "bad result length")
            self.assert_ (str (buff) == "Hello", "bad result returned")
            self.assert_ (partial == 0, "partial result only")
            os.kill (p.pid (), signal.SIGKILL)

class KURLListTestCase (BasePyKDETestCase):
    def runTest (self):
        urls = ["http://slashdot.org", "http://kde.org", "yahoo.com"]
        k = KURL.List ()

        for i in range (len (urls)):
            k.append (KURL (urls [i]))

        self.assert_ (len (k) == len (urls), "__len__ failed")

#        for i in range (len (k)):
#            self.assert_ (str (k [i].url ()) == urls [i], "__getItem__ failed")

        self.assert_ (KURL(urls [2]) in k, "__contains__ failed")
        self.assert_ (KURL ('riverbankcomputing.co.uk') not in k, "__contains__ failed")


class KIOSlaveTestCase (BasePyKDETestCase):
    def runTest (self):
        slave, err, errText = KIO.Slave.createSlave ("http", KURL("http://www.kde.org"))
        self.assert_ (slave, "slave not created")
        self.assert_ (err == 0, "slave returned error")
        self.assert_ (str (slave.protocol ()) == "http", "wrong protocol")
        slave.kill ()


class KIONetRCAutoLoginTestCase (BasePyKDETestCase):
    def runTest (self):
        strings  = ["one", "two", "three"]
        strcmp   = list (strings)
        qstrings = []
        d  = {}
        for s0 in strings:
            slist = QStringList ()
            for s in strings:
                slist.append (" ".join ([s0, s]))

            qstrings .append (QString (s0))
            d [qstrings [-1]] = slist

        al = KIO.NetRC.AutoLogin ()
        al.type     = "type"
        al.machine  = "machine"
        al.login    = "login"
        al.password = "password"
        al.macdef   = d

        self.assert_ (str (al.type) == "type", "string value mismatch - type")
        self.assert_ (str (al.machine) == "machine", "string value mismatch - machine")
        self.assert_ (str (al.login) == "login", "string value mismatch - login")
        self.assert_ (str (al.password) == "password", "string value mismatch - password")

        d = al.macdef
        for key in d.keys ():
            skey = str (key)
            self.assert_ (skey in strcmp, "bad key")

            # each key can only occur once
            strcmp.remove (skey)

            for i in range (d [key].count ()):
                self.assert_ (" ".join ([skey, strings [i]]) == str (d [key][i]), "QStringList value mismatch")

        # all keys have been seen - strcmp is empty
        self.assert_ (not strcmp, "missing key")




class KIOglobalTestCase (BasePyKDETestCase):
    def runTest (self):
        self.nlist    = ["zero", "one", "two", "three"]
        self.nlistcmp = list (self.nlist)
        d             = {}
        udsentry      = []
        udsentrylist  = []

        for i in range (len (self.nlist)):
            d [QString (self.nlist [i])] = QString (self.nlist [i].upper ())
            atom          = KIO.UDSAtom ()
            atom.m_str    = QString (self.nlist [i])
            atom.m_long   = -i
            atom.m_uds    = i

            udsentry.append (atom)
            udsentrylist.append (list (udsentry))

        retD = testKIOMetaData (d)
        for key in retD.keys ():
            k = str (key)
            self.assert_ (k.upper () == str (retD [key]), "key, value mismatch")
            self.assert_  (k in self.nlistcmp, "bad key value")
            self.nlistcmp.remove (k)

        self.nlistcmp = list (self.nlist)
        retUDSEntry = testKIOUDSEntry (udsentry)
        self.testList (retUDSEntry)


        retUDSEntryList = testKIOUDSEntryList (udsentrylist)
        i = 1
        for ulist in retUDSEntryList:
            self.nlistcmp = list (self.nlist [:i])
            i += 1
            self.testList (ulist)


    def testList (self, tlist):
        for item in tlist:
            ms = str (item.m_str)
            self.assert_ (ms in self.nlistcmp, "bad m_str value")
            self.nlistcmp.remove (ms)
            self.assert_ (self.nlist.index (ms) == -item.m_long, "m_long mismatch")
            self.assert_ (self.nlist.index (ms) == item.m_uds, "m_uds mismatch")


class KActionCollectionTestCase (BasePyKDETestCase):
    def runTest (self):
        acts = ["openNew", "open", "save", "saveAs", "print", "quit"]

        ac = mainWindow.actionCollection ()
        for action in ac.actions ():
            self.assert_ (action.name () in acts, "action name mismatch")
            acts.remove (action.name ())


class KActionListTestCase (BasePyKDETestCase):
    def runTest (self):
        acts = ["openNew", "open", "save", "saveAs", "print", "quit"]

        actionList = testKActionList (mainWindow.actionCollection ().actions ())

        for action in actionList:
            self.assert_ (action.name () in acts, "action name mismatch")
            acts.remove (action.name ())


class KEntryMapTestCase (BasePyKDETestCase):
    def runTest (self):
        ke  = []
        kek = []

        ke.append (KEntry ())
        ke [0].mValue     = "value0"
        ke [0].bDirty     = 0
        ke [0].bNLS       = 0
        ke [0].bGlobal    = 0
        ke [0].bImmutable = 0
        ke [0].bDeleted   = 0

        ke.append (KEntry ())
        ke [1].mValue     = "value1"
        ke [1].bDirty     = 1
        ke [1].bNLS       = 1
        ke [1].bGlobal    = 1
        ke [1].bImmutable = 1
        ke [1].bDeleted   = 1


    #   bExpand KDE >= 3.1.0

        kek.append (KEntryKey ())
        kek [0].mGroup   = "group0"
        kek [0].mKey     = "key0"
        kek [0].bLocal   = 1
        kek [0].bDefault = 1
        kek [0].c_key    = "c_key0"


        kek.append (KEntryKey ())
        kek [1].mGroup   = "group1"
        kek [1].mKey     = "key1"
        kek [1].bLocal   = 0
        kek [1].bDefault = 0
        kek [1].c_key    = "c_key1"

        kdictIn = {}
        kdictIn [kek [0]] = ke [0]
        kdictIn [kek [1]] = ke [1]

        kdictOut = testKEntryMap (kdictIn)

        for key in kdictOut.keys ():
            n = int (str (key.mGroup) [-1])
            self.assert_ (0 <= n < 2)
            self.assert_ (kek [n] != 0, "KEntryKey already seen")
            self.assert_ (str (key.mKey) == str (kek [n].mKey), "mKey mismatch")
            self.assert_ (key.bLocal == kek [n].bLocal, "bLocal mismatch")
            self.assert_ (key.bDefault == kek [n].bDefault, "bDefault mismatch")
            self.assert_ (key.c_key == kek [n].c_key, "c_key mismatch")
            keOut = kdictOut [key]
            self.assert_ (str (keOut.mValue) == str (ke [n].mValue), "mValue mismatch")
            self.assert_ (keOut.bDirty == ke [n].bDirty, "bDirty mismatch")
            self.assert_ (keOut.bNLS == ke [n].bNLS, "bNLS mismatch")
            self.assert_ (keOut.bGlobal == ke [n].bGlobal, "bGlobal mismatch")
            self.assert_ (keOut.bImmutable == ke [n].bImmutable, "bImmutable mismatch")
            self.assert_ (keOut.bDeleted == ke [n].bDeleted, "bDeleted mismatch")
            kek [n] = 0






# ------------------- main ----------------------------------

if __name__ == "__main__":
    description = "Automatically test some PyKDE code"
    version     = "1.0"
    aboutData   = KAboutData ("", "",\
        version, description, KAboutData.License_GPL,\
        "(C) 2003 Jim Bublitz")

    KCmdLineArgs.init (sys.argv, aboutData)

    app = KApplication ()
    mainWindow = MainWin (None, "main window")
    mainWindow.show()
    app.exec_loop()


