"""
Copyright 2003 Jim Bublitz

Terms and Conditions

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
COPYRIGHT HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

Except as contained in this notice, the name of the copyright holder shall
not be used in advertising or otherwise to promote the sale, use or other
dealings in this Software without prior written authorization from the
copyright holder.
"""

import sys, os, signal, unittest

from kdecore import KApplication, KCmdLineArgs, KAboutData, KStandardDirs, KGlobal
from kdeui import KMainWindow, KColor, KCommandHistory, KDateValidator, KEdit, KEditListBox
from kfile import KFile

from qt import QString, QValidator, QDir, QPushButton, SIGNAL

False = 0
True  = 1

# ------------------- auxiliary classes ----------------------------------

class MainWin (KMainWindow):
    def __init__ (self, *args):
        apply (KMainWindow.__init__, (self,) + args)

        self.setGeometry (10, 10, 400, 400)

        self.edit = KEdit (self)
        self.edit.setGeometry (10, 10, 220, 150)

        textList = ["Now is the winter of our discontent",\
                    "made glorious summer by this sun of York;",\
                    "and all the clouds that lour'd upon our house",\
                    "in the deep bosom of the ocean buried."]

        for line in textList:
            self.edit.insertLine (line)

        self.editListBox = KEditListBox (self)
        self.editListBox.setGeometry (10, 180, 220, 200)

        self.testBtn = QPushButton ("test", self)
        self.testBtn.setGeometry (290, 355, 60, 22)
        self.connect (self.testBtn, SIGNAL ("clicked ()"), self.doTest)


    def doTest (self):
        suite = unittest.TestSuite ()
        suite.addTest (ULongLongTestCase ('runTest'))
        suite.addTest (DCOPClientTestCase ("runTest"))
        suite.addTest (KApplicationTestCase ("runTest"))
        suite.addTest (KCharsetsTestCase ("runTest"))
        suite.addTest (KColorTestCase ("runTest"))
        suite.addTest (KCommandHistoryTestCase ("runTest"))
        suite.addTest (KDateValidatorTestCase ("runTest"))
        suite.addTest (KEditTestCase ("runTest"))
        suite.addTest (KEditListBoxTestCase ("runTest"))
        suite.addTest (KLocaleTestCase ("runTest"))
        suite.addTest (KFileTestCase ("runTest"))

        runner = unittest.TextTestRunner(sys.stdout, verbosity=2)
        print
        print
        print "testing ..."
        print
        result = runner.run(suite)
        print
        print

        sys.exit (0)

# ------------------- test case classes ----------------------------------

class BasePyKDETestCase (unittest.TestCase):
    pass
#    def setUp (self):
#        description = "A basic application template"
#        version     = "1.0"
#        aboutData   = KAboutData ("", "",\
#            version, description, KAboutData.License_GPL,\
#            "(C) 2003 whoever the author is")

#        aboutData.addAuthor ("author1", "whatever they did", "email@somedomain")
#        aboutData.addAuthor ("author2", "they did something else", "another@email.address")

#        KCmdLineArgs.init (sys.argv, aboutData)

#        KCmdLineArgs.addCmdLineOptions ([("+files", "File to open")])

#        self.app = KApplication ()
#        self.mainWindow = MainWin (None, "main window")
#        mainWindow.show()
#        app.exec_loop()

class ULongLongTestCase (BasePyKDETestCase):
    def runTest (self):
        """
        Test some simple conversions from unsigned long long to QString
        using %MappedType ulonglong
        """
        from kio import KIO
        self.assert_ (KIO.number (0).compare (QString ("0")) == 0, "number (0) failed")
        self.assert_ (KIO.convertSize (0).compare (QString ("0 B")) == 0, "convertSize (0) failed")
        self.assert_ (KIO.number (1).compare (QString ("1")) == 0, "number (1) failed")
        self.assert_ (KIO.convertSize (1).compare (QString ("1 B")) == 0, "convertSize (1) failed")
        self.assert_ (KIO.number (1000).compare (QString ("1000")) == 0, "number (1000) failed")
        self.assert_ (KIO.convertSize (1000).compare (QString ("1,000 B")) == 0, "convertSize (1000) failed")
        self.assert_ (KIO.number (2048).compare (QString ("2048")) == 0, "number (2048) failed")
        self.assert_ (KIO.convertSize (2048).compare (QString ("2.0 KB")) == 0, "convertSize (2048) failed")
        self.assert_ (KIO.number (1234567890123456789).compare (QString ("1234567890123456789")) == 0,\
                      "number (1234567890123456789) failed")
        self.assert_ (KIO.convertSize (1234567890123456789).compare (QString ("1,122,833.0 TB")) == 0,\
                      "convertSize (1234567890123456789) failed")


class DCOPClientTestCase (BasePyKDETestCase):
    def runTest (self):
        """
        Tests both member code (remoteObjects, remoteInterfaces, remoteFunctions)
        but also tests mapped type QCStringList (all lists in this test case
        are QCStringLists)

        Uses KApplication's dcopClient, so doesn't need to import dcop module
        directly
        """

        d = KApplication.dcopClient ()
        self.assert_ (d.isRegistered () == 1)
        regApps = d.registeredApplications ()
        self.assert_ (len (regApps), "registerApplications -- empty list")
        self.assert_ (self.checkListForValue (regApps, "klauncher"), "registerApplciations -- no klauncher")

        remObjs, ok = d.remoteObjects ("klauncher")
        self.assert_ (ok, "remoteObjects returned not ok")
        self.assert_ (len (remObjs), "remoteObjects -- empty list")
        self.assert_ (self.checkListForValue (remObjs, "klauncher"), "remoteObjects -- no klauncher")

        ifs, ok = d.remoteInterfaces ("klauncher", "klauncher")
        self.assert_ (ok, "remoteInterfaces returned not ok")
        self.assert_ (len (ifs), "remoteInterfaces -- empty list")
        self.assert_ (self.checkListForValue (ifs, "DCOPObject"), "remoteInterfaces -- no DCOPObject")

        fcns, ok = d.remoteFunctions ("klauncher", "klauncher")
        self.assert_ (ok, "remoteFunctions returned not ok")
        self.assert_ (len (fcns), "remoteFunctions -- empty list")
        self.assert_ (self.checkListForValue (fcns, "QCStringList interfaces()"), "remoteFunctions -- no QCStringList interfaces()")


    def checkListForValue (self, list, value):
        flag = False
        for item in list:
           flag = str (item) == value
           if flag: break

        return flag


class KApplicationTestCase (BasePyKDETestCase):
    def runTest (self):
        result, error, dcopService, pid = KApplication.startServiceByName ("KWrite", QString.null, "kwrite")
        self.assert_ (result == 0, "startServiceByName returned an error code")
        self.assert_ (str(error) == "", "startServiceByName returned an error string")
        self.assert_ (str(dcopService) == "kwrite-" + str (pid), "startServiceByName returen incorrect dcopService")
        self.assert_ (pid, "startServiceByName returned pid == 0")
        if pid: os.kill (pid, signal.SIGKILL)

        result, error, dcopService, pid = KApplication.startServiceByDesktopName ("kwrite", QString.null, "kwrite")
        self.assert_ (result == 0, "startServiceByDesktopName returned an error code")
        self.assert_ (str(error) == "", "startServiceByDesktopName returned an error string")
        self.assert_ (str(dcopService) == "kwrite-" + str (pid), "startServiceByDesktopName returen incorrect dcopService")
        self.assert_ (pid, "startServiceByDesktopName returned pid == 0")
        if pid: os.kill (pid, signal.SIGKILL)

        stddir = KStandardDirs ()
        kwPath = stddir.findResource ("apps", "Editors/kwrite.desktop")
        result, error, dcopService, pid = KApplication.startServiceByDesktopPath (kwPath, QString.null, "kwrite")
        self.assert_ (result == 0, "startServiceByDesktopPath returned an error code")
        self.assert_ (str(error) == "", "startServiceByDesktopPath returned an error string")
        self.assert_ (str(dcopService) == "kwrite-" + str (pid), "startServiceByDesktopPath returen incorrect dcopService")
        self.assert_ (pid, "startServiceByDesktopPath returned pid == 0")
        if pid: os.kill (pid, signal.SIGKILL)

        result, error, pid = KApplication.kdeinitExec ('kwrite')
        self.assert_ (result == 0, "kdeinitExec returned an error code")
        self.assert_ (str(error) == "", "kdeinitExec returned an error string")
        self.assert_ (pid, "kdeinitExec returned pid == 0")
        if pid: os.kill (pid, signal.SIGKILL)


class KCharsetsTestCase (BasePyKDETestCase):
    def runTest (self):
        cs = KGlobal.charsets ()
        tc, ok = cs.codecForName2 ("Big5")
        self.assert_ (ok, "codecForName2 returned not ok")
        self.assert_ (tc, "codecForName2 returned null")
        self.assert_ (str (tc.name ()) == "Big5", "codecForName2 returned incorrect codec")


class KColorTestCase (BasePyKDETestCase):
    def runTest (self):
        crgb = KColor ()
        r    = 64
        g    = 128
        b    = 255
        crgb.setRgb (r, g, b)
        r1, g1, b1 = crgb.rgb ()

        chsv = KColor ()
        h    = 10
        s    = 20
        v    = 30
        chsv.setHsv (h, s, v)

        h1, s1, v1 = chsv.hsv ()

        self.assert_ (r == r1, "rgb returned incorrect r value")
        self.assert_ (g == g1, "rgb returned incorrect g value")
        self.assert_ (b == b1, "rgb returned incorrect b value")
        self.assert_ (h == h1, "hsv returned incorrect h value")
        self.assert_ (s == s1, "hsv returned incorrect s value")
        self.assert_ (v == v1, "hsv returned incorrect v value")


class KCommandHistoryTestCase (BasePyKDETestCase):
    def runTest (self):
        ch = KCommandHistory ()
        self.assert_ (ch.undoLimit (), "null undo limit")
        self.assert_ (ch.redoLimit (), "null redo limit")

        ch.setUndoLimit (30)
        ch.setRedoLimit (10)
        self.assert_ (ch.undoLimit () == 30, "incorrect undo limit")
        self.assert_ (ch.redoLimit () == 10, "incorrect redo limit")


class KDateValidatorTestCase (BasePyKDETestCase):
    def runTest (self):
        dv = KDateValidator ()
        state, pos = dv.validate ("1-4-03")
        self.assert_ (state == QValidator.Acceptable, "date validator error")
        state, pos = dv.validate ("1-")
        self.assert_ (state == QValidator.Intermediate, "date validator error")


class KEditTestCase (BasePyKDETestCase):
    def runTest (self):
        row, col = mainWindow.edit.posToRowCol (50)
        self.assert_ (row == 1, "incorrect row")
        self.assert_ (col == 14, "incorrect col")


class KEditListBoxTestCase (BasePyKDETestCase):
    def runTest (self):
        itemList = ["one", "two", "three", "four"]
        mainWindow.editListBox.insertStrList (itemList, -1)
        retrList = mainWindow.editListBox.items ()
        self.assert_ (len (retrList) == len (itemList), "item list wrong size")
        for i in range (len (retrList)):
            self.assert_ (str(retrList [i]) == itemList [i], "incorrect item value")

class KLocaleTestCase (BasePyKDETestCase):
    def runTest (self):
        locale = KGlobal.locale ()
        rm, ok = locale.readMoney ("$12.34")
        self.assert_ (ok, "readMoney returned not ok")
        self.assert_ (rm == 12.34, "readMoney returned bad value")

        rn, ok = locale.readNumber ("1,234,567")
        self.assert_ (ok, "readNumber returned not ok")
        self.assert_ (rn == 1234567.0, "readNumber returned bad value")

        qd, ok = locale.readDate (QString ("2003-10-10"))
        self.assert_ (qd.isValid (), "QDate is invalid")
        self.assert_ (ok, "readDAte returned not ok")
        self.assert_ (qd.day () == 10, "wrong day")
        self.assert_ (qd.month (), "wring month")
        self.assert_ (qd.year (), "wrong year")

        qtm, ok = locale.readTime ("11:12:13")
        self.assert_ (ok, "readTime returned not ok")
        self.assert_ (qtm.isValid (), "QTime is invalid")
        self.assert_ (qtm.hour () == 11, "wrong hour")
        self.assert_ (qtm.minute () == 12, "wrong minute")
        self.assert_ (qtm.second () == 13, "wrong second")


class KFileTestCase (BasePyKDETestCase):
    def runTest (self):
        self.assert_ (KFile.isSortByName (QDir.Name))
        self.assert_ (KFile.isSortBySize (QDir.Size))
        self.assert_ (KFile.isSortByDate (QDir.Time))
        self.assert_ (KFile.isSortDirsFirst (QDir.DirsFirst))
        self.assert_ (KFile.isSortCaseInsensitive (QDir.IgnoreCase))
        self.assert_ (KFile.isDefaultView (KFile.Default))
        self.assert_ (KFile.isSimpleView (KFile.Simple))
        self.assert_ (KFile.isDetailView (KFile.Detail))
        self.assert_ (KFile.isSeparateDirs (KFile.SeparateDirs))
        self.assert_ (KFile.isPreviewContents (KFile.PreviewContents))

        self.assert_ (not KFile.isSortByName (QDir.Size))
        self.assert_ (not KFile.isSortBySize (QDir.Name))
        self.assert_ (not KFile.isSortByDate (QDir.Size))
        self.assert_ (not KFile.isSortDirsFirst (QDir.Name))
        self.assert_ (not KFile.isSortCaseInsensitive (QDir.Name))
        self.assert_ (not KFile.isDefaultView (KFile.Detail))
        self.assert_ (not KFile.isSimpleView (KFile.Default))
        self.assert_ (not KFile.isDetailView (KFile.Simple))
        self.assert_ (not KFile.isSeparateDirs (KFile.Detail))
        self.assert_ (not KFile.isPreviewContents (KFile.SeparateDirs))

if __name__ == "__main__":
    description = "Automatically test some PyKDE code"
    version     = "1.0"
    aboutData   = KAboutData ("", "",\
        version, description, KAboutData.License_GPL,\
        "(C) 2003 Jim Bublitz")

    KCmdLineArgs.init (sys.argv, aboutData)

    app = KApplication ()
    mainWindow = MainWin (None, "main window")
    mainWindow.show()
    app.exec_loop()


